@@ -1347,6 +1347,15 @@ get_goption_context (TerminalOptions *options)
       unsupported_option_callback,
       NULL, NULL
     },
+    {
+      "no-environment",
+      0,
+      0,
+      G_OPTION_ARG_NONE,
+      &options->no_environment,
+      N_("Do not pass the environment"),
+      NULL
+    },
     {
       "preferences",
       0,
